@@ -1,3 +1,5 @@
+import copy
+
 import dask.array as da
 import dask.dataframe as dd
 import numpy as np
@@ -11,7 +13,9 @@ from xrspatial import zonal_crosstab as crosstab
 from xrspatial import zonal_stats as stats
 from xrspatial.zonal import regions
 
-from .general_checks import create_test_raster, general_output_checks, has_cuda_and_cupy
+from .general_checks import (
+    assert_input_data_unmodified, create_test_raster, general_output_checks, has_cuda_and_cupy
+)
 
 
 @pytest.fixture
@@ -280,7 +284,58 @@ def result_nodata_values_crosstab_3d():
     return nodata_values, layer, zone_ids, expected_result
 
 
-def check_results(backend, df_result, expected_results_dict):
+@pytest.fixture
+def qgis_zonal_stats():
+    qgis_result = {
+        'zone': [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0],
+        'mean': [748.04910278,
+                 619.62845612,
+                 363.29403178,
+                 582.55223301,
+                 356.15832265,
+                 730.03720856,
+                 468.15884018,
+                 388.61296272,
+                 706.54189046,
+                 677.92201742],
+        'max': [999.14184570,
+                859.26989746,
+                752.95483398,
+                845.27789307,
+                704.23699951,
+                977.11694336,
+                870.53448486,
+                721.16333008,
+                990.82781982,
+                984.69262695],
+        'min': [496.95635986,
+                76.49687195,
+                151.49211121,
+                290.60409546,
+                51.21858978,
+                447.12411499,
+                49.61272812,
+                32.27882004,
+                468.97912598,
+                242.24084473],
+        'sum': [1496.09820557,
+                3717.77073669,
+                1089.88209534,
+                4077.86563110,
+                3205.42490387,
+                2920.14883423,
+                2340.79420090,
+                2331.67777634,
+                2119.62567139,
+                2033.76605225],
+        'count': [2, 6, 3, 7, 9, 4, 5, 6, 3, 3]
+    }
+    return qgis_result
+
+
+def check_results(
+        backend, df_result, expected_results_dict, rtol=1e-05, atol=1e-07, equal_nan=True
+):
     if 'dask' in backend:
         # dask case, compute result
         assert isinstance(df_result, dd.DataFrame)
@@ -292,21 +347,35 @@ def check_results(backend, df_result, expected_results_dict):
     assert (df_result['zone'] == expected_results_dict['zone']).all()
     # stats columns
     for col in df_result.columns[1:]:
-        np.testing.assert_allclose(df_result[col], expected_results_dict[col])
+        np.testing.assert_allclose(
+            df_result[col], expected_results_dict[col], rtol=rtol, atol=atol, equal_nan=equal_nan
+        )
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy', 'cupy'])
 def test_default_stats(backend, data_zones, data_values_2d, result_default_stats):
     if backend == 'cupy' and not has_cuda_and_cupy():
         pytest.skip("Requires CUDA and CuPy")
+
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
     df_result = stats(zones=data_zones, values=data_values_2d)
     check_results(backend, df_result, result_default_stats)
 
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
+
 
 @pytest.mark.parametrize("backend", ['numpy'])
 def test_default_stats_dataarray(
     backend, data_zones, data_values_2d, result_default_stats_dataarray
 ):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
     dataarray_result = stats(
         zones=data_zones, values=data_values_2d, return_type='xarray.DataArray'
     )
@@ -317,22 +386,34 @@ def test_default_stats_dataarray(
         verify_dtype=False,
         verify_attrs=False,
     )
-
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy', 'cupy'])
 def test_zone_ids_stats(backend, data_zones, data_values_2d, result_zone_ids_stats):
     if backend == 'cupy' and not has_cuda_and_cupy():
         pytest.skip("Requires CUDA and CuPy")
+
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
     zone_ids, expected_result = result_zone_ids_stats
     df_result = stats(zones=data_zones, values=data_values_2d,
                       zone_ids=zone_ids)
     check_results(backend, df_result, expected_result)
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
 
 
 @pytest.mark.parametrize("backend", ['numpy'])
 def test_zone_ids_stats_dataarray(
     backend, data_zones, data_values_2d, result_zone_ids_stats_dataarray
 ):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
     zone_ids, expected_result = result_zone_ids_stats_dataarray
     dataarray_result = stats(
         zones=data_zones, values=data_values_2d, zone_ids=zone_ids, return_type='xarray.DataArray'
@@ -340,6 +421,8 @@ def test_zone_ids_stats_dataarray(
     general_output_checks(
         data_values_2d, dataarray_result, expected_result, verify_dtype=False, verify_attrs=False
     )
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'cupy'])
@@ -348,6 +431,10 @@ def test_custom_stats(backend, data_zones, data_values_2d, result_custom_stats):
     if backend == 'cupy' and not has_cuda_and_cupy():
         pytest.skip("Requires CUDA and CuPy")
 
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
     custom_stats = {
         'double_sum': _double_sum,
         'range': _range,
@@ -359,10 +446,15 @@ def test_custom_stats(backend, data_zones, data_values_2d, result_custom_stats):
         zone_ids=zone_ids, nodata_values=nodata_values
     )
     check_results(backend, df_result, expected_result)
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
 
 
 @pytest.mark.parametrize("backend", ['numpy'])
 def test_custom_stats_dataarray(backend, data_zones, data_values_2d, result_custom_stats_dataarray):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
     # ---- custom stats returns a xr.DataArray (NumPy only) ----
     custom_stats = {
         'double_sum': _double_sum,
@@ -376,43 +468,95 @@ def test_custom_stats_dataarray(backend, data_zones, data_values_2d, result_cust
     general_output_checks(
         data_values_2d, dataarray_result, expected_result, verify_dtype=False, verify_attrs=False
     )
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
+
+
+def test_zonal_stats_against_qgis(elevation_raster_no_nans, raster, qgis_zonal_stats):
+    stats_funcs = list(set(qgis_zonal_stats.keys()) - set(['zone']))
+    zones_agg = create_test_raster(raster)
+    values_agg = create_test_raster(elevation_raster_no_nans)
+
+    xrspatial_df_result = stats(
+        zones=zones_agg, values=values_agg, stats_funcs=stats_funcs
+    )
+    check_results('numpy', xrspatial_df_result, qgis_zonal_stats, atol=1e-5)
+
+
+@pytest.mark.parametrize("backend", ['numpy', 'dask+numpy', 'cupy'])
+def test_zonal_stats_inputs_unmodified(backend, data_zones, data_values_2d, result_default_stats):
+    if backend == 'cupy' and not has_cuda_and_cupy():
+        pytest.skip("Requires CUDA and CuPy")
+
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
+    df_result = stats(zones=data_zones, values=data_values_2d)
+
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy'])
 def test_count_crosstab_2d(backend, data_zones, data_values_2d, result_count_crosstab_2d):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
     zone_ids, cat_ids, expected_result = result_count_crosstab_2d
     df_result = crosstab(
         zones=data_zones, values=data_values_2d, zone_ids=zone_ids, cat_ids=cat_ids,
     )
     check_results(backend, df_result, expected_result)
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy'])
 def test_percentage_crosstab_2d(backend, data_zones, data_values_2d, result_percentage_crosstab_2d):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_2d = copy.deepcopy(data_values_2d)
+
     nodata_values, zone_ids, cat_ids, expected_result = result_percentage_crosstab_2d
     df_result = crosstab(
         zones=data_zones, values=data_values_2d, zone_ids=zone_ids, cat_ids=cat_ids,
         nodata_values=nodata_values, agg='percentage'
     )
     check_results(backend, df_result, expected_result)
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_2d, copied_data_values_2d)
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy'])
 def test_crosstab_3d_count(backend, data_zones, data_values_3d, result_crosstab_3d):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_3d = copy.deepcopy(data_values_3d)
+
     layer, zone_ids, expected_result = result_crosstab_3d
     df_result = crosstab(zones=data_zones, values=data_values_3d,
                          zone_ids=zone_ids, layer=layer, agg='count')
     check_results(backend, df_result, expected_result['count'])
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_3d, copied_data_values_3d)
 
 
 @pytest.mark.parametrize("backend", ['numpy'])
 def test_crosstab_3d_agg_method(backend, data_zones, data_values_3d, result_crosstab_3d):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_3d = copy.deepcopy(data_values_3d)
+
     layer, zone_ids, expected_result = result_crosstab_3d
     agg_methods = ['min', 'max', 'mean', 'sum', 'std', 'var', 'count']
     for agg in agg_methods:
         df_result = crosstab(zones=data_zones, values=data_values_3d,
                              zone_ids=zone_ids, layer=layer, agg=agg)
         check_results(backend, df_result, expected_result[agg])
+        assert_input_data_unmodified(data_zones, copied_data_zones)
+        assert_input_data_unmodified(data_values_3d, copied_data_values_3d)
 
 
 @pytest.mark.parametrize("backend", ['numpy', 'dask+numpy'])
@@ -422,12 +566,18 @@ def test_nodata_values_crosstab_3d(
     data_values_3d,
     result_nodata_values_crosstab_3d
 ):
+    # copy input data to verify they're unchanged after running the function
+    copied_data_zones = copy.deepcopy(data_zones)
+    copied_data_values_3d = copy.deepcopy(data_values_3d)
+
     nodata_values, layer, zone_ids, expected_result = result_nodata_values_crosstab_3d
     df_result = crosstab(
         zones=data_zones, values=data_values_3d, zone_ids=zone_ids,
         layer=layer, nodata_values=nodata_values
     )
     check_results(backend, df_result, expected_result)
+    assert_input_data_unmodified(data_zones, copied_data_zones)
+    assert_input_data_unmodified(data_values_3d, copied_data_values_3d)
 
 
 def test_apply():
